@@ -7,6 +7,8 @@ import logging
 import os
 import click
 
+import pandas as pd
+
 import yaml
 from linkml_runtime.linkml_model import SchemaDefinition
 from oaklib.selector import get_resource_from_shorthand, get_implementation_from_shorthand
@@ -14,11 +16,14 @@ from oaklib.selector import get_resource_from_shorthand, get_implementation_from
 from schema_automator import JsonLdAnnotator
 from schema_automator.annotators.schema_annotator import SchemaAnnotator
 from schema_automator.generalizers.csv_data_generalizer import CsvDataGeneralizer
+from schema_automator.generalizers.generalizer import DEFAULT_CLASS_NAME, DEFAULT_SCHEMA_NAME
+from schema_automator.generalizers.pandas_generalizer import PandasDataGeneralizer
 from schema_automator.importers.dosdp_import_engine import DOSDPImportEngine
 from schema_automator.generalizers.json_instance_generalizer import JsonDataGeneralizer
 from schema_automator.importers.jsonschema_import_engine import JsonSchemaImportEngine
 from schema_automator.importers.owl_import_engine import OwlImportEngine
 from schema_automator.generalizers.rdf_data_generalizer import RdfDataGeneralizer
+from schema_automator.importers.sql_import_engine import SqlImportEngine
 from schema_automator.utils.schemautils import minify_schema, write_schema
 
 input_option = click.option(
@@ -34,9 +39,13 @@ output_option = click.option(
 schema_name_option = click.option(
     '--schema-name',
     '-n',
-    default='example',
+    default=DEFAULT_SCHEMA_NAME,
     show_default=True,
     help='Schema name')
+annotator_option = click.option(
+    '--annotator',
+    '-A',
+    help='name of annotator to use for auto-annotating results. Must be an OAK selector')
 
 
 @click.group()
@@ -50,28 +59,37 @@ def main(verbose: int, quiet: bool):
 
     A subcommand must be passed, for example:
 
-        schemauto -v SUBCOMMAND [OPTIONS] ARGUMENTS
+        schemauto SUBCOMMAND [OPTIONS] ARGUMENTS
+
+    To see logging or debugging info, the verbosity
+    flag should be specified BEFORE the subcommand:
+
+        schemauto -vv SUBCOMMAND [OPTIONS] ARGUMENTS
     """
+    logger = logging.getLogger()
     if verbose >= 2:
-        logging.basicConfig(level=logging.DEBUG)
+        logger.setLevel(logging.DEBUG)
     elif verbose == 1:
-        logging.basicConfig(level=logging.INFO)
+        logger.setLevel(logging.INFO)
     else:
-        logging.basicConfig(level=logging.WARNING)
+        logger.setLevel(logging.WARNING)
     if quiet:
-        logging.basicConfig(level=logging.ERROR)
+        logger.setLevel(logging.ERROR)
+    logging.info(f"Log level={verbose}")
 
 
 @main.command()
 @click.argument('tsvfile')  # input TSV (must have column headers
 @output_option
 @schema_name_option
-@click.option('--class-name', '-c', default='example', help='Core class name in schema')
+@annotator_option
+@click.option('--class-name', '-c', default=DEFAULT_CLASS_NAME, help='Core class name in schema')
 @click.option('--column-separator', '-s', default='\t', help='separator')
 @click.option('--downcase-header/--no-downcase-header', default=False, help='if true make headers lowercase')
 @click.option('--enum-columns', '-E', multiple=True, help='column that is forced to be an enum')
 @click.option('--robot/--no-robot', default=False, help='set if the TSV is a ROBOT template')
-def generalize_tsv(tsvfile, output, class_name, schema_name, **kwargs):
+@click.option('--pandera/--no-pandera', default=False, help='set to use panderas as inference engine')
+def generalize_tsv(tsvfile, output, class_name, schema_name, pandera: bool, annotator, **kwargs):
     """
     Generalizes from a single TSV file to a single-class schema
 
@@ -81,8 +99,15 @@ def generalize_tsv(tsvfile, output, class_name, schema_name, **kwargs):
 
         schemauto generalize-tsv --class-name Person --schema-name PersonInfo my/data/persons.tsv
     """
-    ie = CsvDataGeneralizer(**kwargs)
+    if pandera:
+        ie = PandasDataGeneralizer(**kwargs)
+    else:
+        ie = CsvDataGeneralizer(**kwargs)
     schema = ie.convert(tsvfile, class_name=class_name, schema_name=schema_name)
+    if annotator:
+        impl = get_implementation_from_shorthand(annotator)
+        sa = SchemaAnnotator(impl)
+        schema = sa.annotate_schema(schema)
     write_schema(schema, output)
 
 
@@ -115,11 +140,45 @@ def generalize_tsvs(tsvfiles, output, schema_name, **kwargs):
     write_schema(schema, output)
 
 
+@main.command()
+@click.argument('url')  # input TSV (must have column headers
+@output_option
+@schema_name_option
+@click.option('--class-name', '-c', default=DEFAULT_CLASS_NAME, help='Core class name in schema')
+@click.option('--pandera/--no-pandera', default=False, help='set to use panderas as inference engine')
+@click.option('--data-output', help='Path to file of downloaded data')
+@click.option('--table-number',
+              type=int,
+              default=0,
+              show_default=True,
+              help='If URL has multiple tables, use this one (zero-based)')
+def generalize_htmltable(url, output, class_name, schema_name, pandera: bool,
+                         table_number: int, data_output,
+                         **kwargs):
+    """
+    Generalizes from a table parsed from a URL
+
+    Uses pandas/beautiful soup
+    """
+    dfs = pd.read_html(url)
+    logging.info(f"{url} has {len(dfs)} tables")
+    df = dfs[table_number]
+    if data_output:
+        df.to_csv(data_output, sep="\t")
+    if pandera:
+        ie = PandasDataGeneralizer(**kwargs)
+        schema = ie.convert(df, class_name=class_name, schema_name=schema_name)
+    else:
+        ie = CsvDataGeneralizer(**kwargs)
+        schema = ie.convert_dicts(df.to_dict('records'))
+    write_schema(schema, output)
+
+
 @main.command()
 @click.argument('dpfiles', nargs=-1) ## input DOSDPs
 @output_option
 @schema_name_option
-@click.option('--range-as-enum/--no-range-as-enums',
+@click.option('--range-as-enums/--no-range-as-enums',
               default=True,
               help="Model range ontology classes as enums")
 def import_dosdps(dpfiles, output, **args):
@@ -127,12 +186,31 @@ def import_dosdps(dpfiles, output, **args):
     Imports DOSDP pattern YAML to a LinkML schema
 
     See :ref:`importers` for more on the importers framework
+
+    Example:
+
+        schemauto import-dosdps --range-as-enums patterns/*yaml -o my-schema.yaml
     """
     ie = DOSDPImportEngine()
     schema = ie.convert(dpfiles, **args)
     write_schema(schema, output)
 
 
+@main.command()
+@click.argument('db')
+@output_option
+@schema_name_option
+def import_sql(db, output, **args):
+    """
+    Imports a schema by introspecting a relational database
+
+    See :ref:`importers` for more on the importers framework
+    """
+    ie = SqlImportEngine()
+    schema = ie.convert(db, **args)
+    write_schema(schema, output)
+
+
 @main.command()
 @click.argument('input')
 @output_option
@@ -152,13 +230,38 @@ def generalize_json(input, output, schema_name, format, omit_null, **kwargs):
 
     Example:
 
-        schemauto generalize-json my/data/persons.json
+        schemauto generalize-json my/data/persons.json -o my.yaml
     """
     ie = JsonDataGeneralizer(omit_null=omit_null)
     schema = ie.convert(input, format=format, **kwargs)
     write_schema(schema, output)
 
 
+@main.command()
+@click.argument('input')
+@output_option
+@schema_name_option
+@click.option('--container-class-name', default='Container', help="name of root class")
+@click.option('--enum-columns', '-E', multiple=True, help='column(s) that is forced to be an enum')
+@click.option('--enum-mask-columns', multiple=True, help='column(s) that are excluded from being enums')
+@click.option('--max-enum-size', default=50, help='do not create an enum if more than max distinct members')
+@click.option('--enum-threshold', default=0.1, help='if the number of distinct values / rows is less than this, do not make an enum')
+@click.option('--omit-null/--no-omit-null', default=False, help="if true, ignore null values")
+def generalize_toml(input, output, schema_name, omit_null, **kwargs):
+    """
+    Generalizes from a TOML file to a schema
+
+    See :ref:`generalizers` for more on the generalization framework
+
+    Example:
+
+        schemauto generalize-toml my/data/conf.toml -o my.yaml
+    """
+    ie = JsonDataGeneralizer(omit_null=omit_null)
+    schema = ie.convert(input, format='toml', **kwargs)
+    write_schema(schema, output)
+
+
 @main.command()
 @click.argument('input')
 @output_option
@@ -190,11 +293,15 @@ def import_owl(owlfile, output, **args):
     """
     Import an OWL ontology to LinkML
 
-    Note this works best for "schema-style" ontologies
+    Note:
+         - this works best for "schema-style" ontologies
+         - input must be in functional syntax
 
     See :ref:`importers` for more on the importer framework
 
-    Note: input must be in functional syntax
+    Example:
+
+        schemauto import-owl prov.ofn -o my.yaml
     """
     sie = OwlImportEngine()
     schema = sie.convert(owlfile, **args)
@@ -215,7 +322,7 @@ def generalize_rdf(rdffile, dir, output, **args):
 
     Example:
 
-        schemauto generalize-json my/data/persons.ttl
+        schemauto generalize-rdf my/data/persons.ttl
     """
     sie = RdfDataGeneralizer()
     if not os.path.exists(dir):
@@ -230,18 +337,25 @@ def generalize_rdf(rdffile, dir, output, **args):
               default=False,
               show_default=True,
               help="if set, only use results that are mapped to CURIEs")
-@click.option('--input', '-i', help="OAK input ontology selector")
+@click.option('--input',
+              '-i',
+              help="OAK input ontology selector")
 @output_option
-def annotate_schema(schema: str, input: str, output: str, curie_only: bool, **args):
+def annotate_schema(schema: str, input: str, output: str, **kwargs):
     """
     Annotate all elements of a schema
 
-    Requires Bioportal API key
+    This uses OAK, and you can provide any OAK backend. See:
+
+    `Selectors <https://incatools.github.io/ontology-access-kit/selectors.html>`_
+
+    Note:
+
+        if you provide "bioportal" as selector, you must have set your API key uising OAK
     """
     impl = get_implementation_from_shorthand(input)
-    logging.basicConfig(level=logging.INFO)
-    annr = SchemaAnnotator(impl)
-    schema = annr.annotate_schema(schema, curie_only=curie_only)
+    annr = SchemaAnnotator(impl, **kwargs)
+    schema = annr.annotate_schema(schema)
     write_schema(schema, output)
 
 
@@ -256,8 +370,8 @@ def enrich_schema(schema: str, input: str, output: str, **args):
     Requires Bioportal API key
     """
     impl = get_implementation_from_shorthand(input)
-    logging.basicConfig(level=logging.INFO)
     annr = SchemaAnnotator(impl)
+    logging.info(f"Enriching: {schema}")
     schema = annr.enrich(schema)
     write_schema(schema, output)
 
@@ -269,7 +383,6 @@ def annotate_using_jsonld(schema: str, output: str, **args):
     """
     Annotates a schema using a Json-LD context file
     """
-    logging.basicConfig(level=logging.INFO)
     annr = JsonLdAnnotator()
     schemadef = SchemaDefinition(schema)
     annr.annotate_schema(schemadef)
