@@ -2,6 +2,8 @@ import click
 from typing import Union, Dict, List, Any
 from collections import defaultdict
 import json
+
+import tomlkit
 import yaml
 import gzip
 
@@ -19,12 +21,24 @@ from schema_automator.utils.schemautils import write_schema
 
 @dataclass
 class JsonDataGeneralizer(Generalizer):
+    """
+    A generalizer that abstract from JSON instance data
+    """
     mappings: dict = None
     omit_null: bool = None
 
     def convert(self, input: Union[str, Dict], format: str = 'json',
                 container_class_name='Container',
                 **kwargs) -> SchemaDefinition:
+        """
+        Generalizes from a JSON file
+
+        :param input:
+        :param format:
+        :param container_class_name:
+        :param kwargs:
+        :return:
+        """
         csv_engine = CsvDataGeneralizer(**kwargs)
 
         if isinstance(input, dict):
@@ -41,6 +55,11 @@ class JsonDataGeneralizer(Generalizer):
                     obj = json.load(stream)
                 elif format == 'yaml':
                     obj = yaml.safe_load(stream)
+                elif format == 'toml':
+                    obj_str = "".join(stream.readlines())
+                    toml_obj = tomlkit.parse(obj_str)
+                    json_str = json.dumps(toml_obj)
+                    obj = json.loads(json_str)
                 else:
                     raise Exception(f'bad format {format}')
         rows_by_table = defaultdict(list)
